/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package com.stormacq.demo.nxt;

//jersey
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.UriInfo;
import javax.ws.rs.Consumes;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;


//json
import org.codehaus.jettison.json.*;


/**
 * REST Web Service
 *
 * @author sst
 */

@Path("pilot")
public class PilotResource {

    private Robot robot = null;

    //JSON objects
    private static final String JSON_INITIALIZE = "initialization";
    private static final String JSON_INITIALIZE_WHEELDIAMETER = "wheelDiameter";
    private static final String JSON_INITIALIZE_WHEELDISTANCE = "wheelDistance";
    //private static final String JSON_INITIALIZE_MOTORS = "motors";
    private static final String JSON_COMMANDS_LIST = "commands";
    private static final String JSON_COMMAND = "command";
    private static final String JSON_COMMAND_TRAVEL = "travel";
    private static final String JSON_COMMAND_TRAVEL_ARG = "distance";
    private static final String JSON_COMMAND_TURN = "turn";
    private static final String JSON_COMMAND_TURN_ARG = "angle";
    private static final String JSON_COMMAND_FORWARD = "forward";
    private static final String JSON_COMMAND_FORWARD_ARG = "speed";
    private static final String JSON_COMMAND_BACKWARD = "backward";
    private static final String JSON_COMMAND_BACKWARD_ARG = "speed";
    private static final String JSON_COMMAND_STOP = "stop";
    private static final String JSON_COMMAND_DISCONNECT = "disconnect";

    @Context
    private UriInfo context;

    /** Creates a new instance of PilotResource */
    public PilotResource() {
    }

    /**
     * PUT method for updating or creating an instance of PilotResource
     * @param content representation for the resource
     * @return an HTTP response with content of the updated or created resource.
     */
    @PUT
    @Consumes("application/json")
    public String putJson(String content) {

        String result = "{ \"status\" : \"OK\" }";

        try {
            JSONTokener parser = new JSONTokener(content);
            JSONObject json = new JSONObject(parser);

            //when pilot is not initialized yet, look for initialization order
            robot = Robot.getSingleton();
            if (! robot.isInitiliazed()) {

                JSONObject initiliazeOrder = null;

                try {

                    initiliazeOrder = json.getJSONObject(JSON_INITIALIZE);
                    robot.initialize((float)initiliazeOrder.getDouble(JSON_INITIALIZE_WHEELDIAMETER),
                                     (float)initiliazeOrder.getDouble(JSON_INITIALIZE_WHEELDISTANCE));
                    
                } catch (JSONException e ) {

                    String msg = "MUST send and initialization order first";
                    Logger.getLogger(PilotResource.class.getName()).log(Level.WARNING, msg);
                    result = "{ \"status\" : \"" + msg + "\" }";
                    return result;
                }
            }


            //read JSON commands
            JSONArray commands = null;

            try {

                commands = json.getJSONArray(JSON_COMMANDS_LIST);

                //execute them
                for(int i=0; i<commands.length(); i++) {

                    JSONObject cmd = commands.getJSONObject(i);
                    Logger.getLogger(PilotResource.class.getName()).log(Level.INFO, "Command = {0}", cmd.toString());

                    String order = cmd.getString(JSON_COMMAND);
                    if (JSON_COMMAND_TRAVEL.equalsIgnoreCase(order)) {
                        float distance = (float)cmd.getDouble(JSON_COMMAND_TRAVEL_ARG);
                        robot.travel(distance);
                    } else
                    if (JSON_COMMAND_TURN.equalsIgnoreCase(order)) {
                        float angle = (float)cmd.getDouble(JSON_COMMAND_TURN_ARG);
                        robot.rotate(angle);
                    }
                    if (JSON_COMMAND_FORWARD.equalsIgnoreCase(order)) {
                        float speed = (float)cmd.getDouble(JSON_COMMAND_FORWARD_ARG);
                        robot.stop();
                        robot.setSpeed(speed);
                        robot.forward();
                    }
                    if (JSON_COMMAND_BACKWARD.equalsIgnoreCase(order)) {
                        float speed = (float)cmd.getDouble(JSON_COMMAND_BACKWARD_ARG);
                        robot.stop();
                        robot.setSpeed(speed);
                        robot.backward();
                    }
                    if (JSON_COMMAND_STOP.equalsIgnoreCase(order)) {
                        robot.stop();
                    }
                    if (JSON_COMMAND_DISCONNECT.equalsIgnoreCase(order)) {
                        robot.disconnect();
                        robot = null;
                    }
                }
                
            } catch (JSONException e) {

                String msg = "No command to execute";
                Logger.getLogger(PilotResource.class.getName()).log(Level.INFO, msg);
                result = "{ \"status\" : \"" + msg + "\" }";

            }

        } catch (Exception ex) {
            String msg = "Error whil eparsing JSON request - or - communicating with NXT";
            Logger.getLogger(PilotResource.class.getName()).log(Level.SEVERE, msg, ex);
            result = "{ \"status\" : \"" + msg + "\" }";
        }

        return result;
    }
}
