/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package com.stormacq.demo.nxt;

//lejos
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import lejos.nxt.ColorLightSensor;
import lejos.nxt.remote.NXTCommand;
import lejos.pc.comm.NXTCommLogListener;
import lejos.pc.comm.NXTConnector;
import lejos.pc.comm.NXTComm;

import lejos.nxt.Motor;
import lejos.nxt.SensorPort;
import lejos.robotics.Colors;
import lejos.robotics.navigation.*;


/**
 *
 * @author sst
 */
public class Robot {

    private static Robot singleton = null;

    public static Robot getSingleton() {

        final Object lock = Robot.class;

        if (singleton == null) {
            synchronized(lock) {
                singleton = new Robot();
            }
        }

        return singleton;
    }

    private Robot() {}

    private Pilot pilot = null;
    private static final float ROTATION_ANGLE = 90.0f;

    public void connect() {

        final String connectString = "usb://NXT"; //always use USB as BT will be checked next

        try {
            Logger.getLogger(getClass().getName()).log(Level.INFO, "Initializing communication with NXT");
            NXTConnector cmd = new NXTConnector();
            cmd.addLogListener(new NXTCommLogListener() {

                @Override
                public void logEvent(String message) {
                    Logger.getLogger(getClass().getName()).log(Level.INFO, message);
                }

                @Override
                public void logEvent(Throwable throwable) {
                    Logger.getLogger(getClass().getName()).log(Level.SEVERE, "Exception while talking to NXT brick", throwable);
                }
            });
	    if (!cmd.connectTo(connectString, NXTComm.LCP)) {
	    //if (!cmd.connectTo(NXTComm.PACKET)) {
                Logger.getLogger(getClass().getName()).log(Level.WARNING, "Failed to connect");
                pilot = null;
                return;
            }

            NXTCommand.getSingleton().setNXTComm(cmd.getNXTComm());
            Logger.getLogger(getClass().getName()).log(Level.FINE, "Done");

        } catch (Exception e) {
            Logger.getLogger(getClass().getName()).log(Level.SEVERE, "Exception while initializing connection", e);
            pilot = null;
        }

    }

    public void disconnect() {
        try {
            pilot = null;
            NXTCommand.getSingleton().close();
        } catch (IOException ex) {
            Logger.getLogger(Robot.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public void initialize(float wheelDiameter, float wheelDistance) {
        this.connect();
        pilot = new TachoPilot(wheelDiameter, wheelDistance, Motor.A, Motor.C);
    }

    public boolean isInitiliazed() {
        return pilot != null;
    }

    public void rotateLeft() {
        rotate(0-ROTATION_ANGLE);
    }

    public void rotateRigh() {
        rotate(ROTATION_ANGLE);

    }

    public void rotate(float angle) {
        pilot.rotate(angle);
    }

    public void travel(float distance) {
        pilot.travel(distance);
    }

    public void forward() {
        pilot.forward();
    }

    public void stop() {
        pilot.stop();
    }

    public void backward() {
        pilot.backward();
    }

    public void setSpeed(float speed) {
        pilot.setMoveSpeed(speed);
    }

    public float getSpeed() {
        return pilot.getMoveSpeed();
    }

    public float getMaxSpeed() {
        return pilot.getMoveMaxSpeed();
    }

    public String getDeviceName() throws IOException {
        Logger.getLogger(getClass().getName()).log(Level.INFO, "Getting Device information");
        String result = NXTCommand.getSingleton().getFriendlyName();
        Logger.getLogger(getClass().getName()).log(Level.INFO, "Connected to NXT named : {0}", result);

        return result;
    }

    public int getDeviceBateryLevel() throws IOException {
        Logger.getLogger(getClass().getName()).log(Level.INFO, "Getting Device information");
        int result = NXTCommand.getSingleton().getBatteryLevel();
        Logger.getLogger(getClass().getName()).log(Level.INFO, "Device Batery Level : {0}", result);

        return result;
    }

    public String getDeviceFirmwareVersion() throws IOException {
        Logger.getLogger(getClass().getName()).log(Level.INFO, "Getting Device information");
        String result = NXTCommand.getSingleton().getFirmwareVersion().firmwareVersion;
        Logger.getLogger(getClass().getName()).log(Level.INFO, "Device Batery Level : {0}", result);

        return result;
    }

    private SensorPort getPortWithNumber(int portNumber) {

        SensorPort result;

        switch(portNumber) {
            case 1: result = SensorPort.S1;
                    break;
            case 2: result = SensorPort.S2;
                    break;
            case 3: result = SensorPort.S3;
                    break;
            default:
                result = SensorPort.S1;
        }

        return result;
    }

    public String getColorName(int portNumber) {

        SensorPort port = getPortWithNumber(portNumber);

        Logger.getLogger(getClass().getName()).log(Level.INFO, "Getting Color Name from port {0}", portNumber);
        ColorLightSensor lightSensor = new ColorLightSensor(port, ColorLightSensor.TYPE_COLORFULL);
        Colors.Color color = lightSensor.readColor();

        return color.name();
    }

    public int getColorValue(int portNumber) {

        SensorPort port = getPortWithNumber(portNumber);

        Logger.getLogger(getClass().getName()).log(Level.INFO, "Getting Color Value from port {0}", portNumber);
        ColorLightSensor lightSensor = new ColorLightSensor(port, ColorLightSensor.TYPE_COLORFULL);
        return lightSensor.readValue();
    }

}
